'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _raf2 = require('raf');

var _raf3 = _interopRequireDefault(_raf2);

var _domCss = require('dom-css');

var _domCss2 = _interopRequireDefault(_domCss);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _getScrollbarWidth = require('../utils/getScrollbarWidth');

var _getScrollbarWidth2 = _interopRequireDefault(_getScrollbarWidth);

var _returnFalse = require('../utils/returnFalse');

var _returnFalse2 = _interopRequireDefault(_returnFalse);

var _styles = require('./styles');

var _defaultRenderElements = require('./defaultRenderElements');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

exports.default = (0, _react.createClass)({

    displayName: 'Scrollbars',

    propTypes: {
        onScroll: _react.PropTypes.func,
        renderScrollbarHorizontal: _react.PropTypes.func,
        renderScrollbarVertical: _react.PropTypes.func,
        renderThumbHorizontal: _react.PropTypes.func,
        renderThumbVertical: _react.PropTypes.func,
        renderView: _react.PropTypes.func,
        style: _react.PropTypes.object,
        children: _react.PropTypes.node
    },

    getDefaultProps: function getDefaultProps() {
        return {
            renderScrollbarHorizontal: _defaultRenderElements.defaultRenderScrollbarHorizontal,
            renderScrollbarVertical: _defaultRenderElements.defaultRenderScrollbarVertical,
            renderThumbHorizontal: _defaultRenderElements.defaultRenderThumbHorizontal,
            renderThumbVertical: _defaultRenderElements.defaultRenderThumbVertical,
            renderView: _defaultRenderElements.defaultRenderView
        };
    },
    componentDidMount: function componentDidMount() {
        this.addListeners();
        this.update();
    },
    componentDidUpdate: function componentDidUpdate() {
        this.update();
    },
    componentWillUnmount: function componentWillUnmount() {
        this.removeListeners();
        if (this.timer) _raf3.default.cancel(this.timer);
    },
    getScrollLeft: function getScrollLeft() {
        var view = this.refs.view;

        return view.scrollLeft;
    },
    getScrollTop: function getScrollTop() {
        var view = this.refs.view;

        return view.scrollTop;
    },
    getScrollWidth: function getScrollWidth() {
        var view = this.refs.view;

        return view.scrollWidth;
    },
    getScrollHeight: function getScrollHeight() {
        var view = this.refs.view;

        return view.scrollHeight;
    },
    getWidth: function getWidth() {
        var view = this.refs.view;

        return view.clientWidth;
    },
    getHeight: function getHeight() {
        var view = this.refs.view;

        return view.clientHeight;
    },
    getPosition: function getPosition() {
        var view = this.refs.view;
        var scrollTop = view.scrollTop;
        var scrollLeft = view.scrollLeft;
        var scrollHeight = view.scrollHeight;
        var scrollWidth = view.scrollWidth;
        var clientHeight = view.clientHeight;
        var clientWidth = view.clientWidth;

        return {
            x: scrollLeft * 100 / clientWidth,
            y: scrollTop * 100 / clientHeight,
            left: scrollLeft / (scrollWidth - clientWidth) || 0,
            top: scrollTop / (scrollHeight - clientHeight) || 0,
            scrollLeft: scrollLeft,
            scrollTop: scrollTop,
            scrollWidth: scrollWidth,
            scrollHeight: scrollHeight,
            clientWidth: clientWidth,
            clientHeight: clientHeight
        };
    },
    getInnerSizePercentage: function getInnerSizePercentage() {
        var view = this.refs.view;

        return {
            widthPercentageInner: view.clientWidth * 100 / view.scrollWidth,
            heightPercentageInner: view.clientHeight * 100 / view.scrollHeight
        };
    },
    scrollTop: function scrollTop() {
        var top = arguments.length <= 0 || arguments[0] === undefined ? 0 : arguments[0];
        var view = this.refs.view;

        view.scrollTop = top;
    },
    scrollToTop: function scrollToTop() {
        var view = this.refs.view;

        view.scrollTop = 0;
    },
    scrollToBottom: function scrollToBottom() {
        var view = this.refs.view;

        view.scrollTop = view.scrollHeight;
    },
    scrollLeft: function scrollLeft() {
        var left = arguments.length <= 0 || arguments[0] === undefined ? 0 : arguments[0];
        var view = this.refs.view;

        view.scrollLeft = left;
    },
    scrollToLeft: function scrollToLeft() {
        var view = this.refs.view;

        view.scrollLeft = 0;
    },
    scrollToRight: function scrollToRight() {
        var view = this.refs.view;

        view.scrollLeft = view.scrollWidth;
    },
    addListeners: function addListeners() {
        if (typeof document === 'undefined') return;
        this.refs.view.addEventListener('scroll', this.handleScroll);
        this.refs.barVertical.addEventListener('mousedown', this.handleVerticalTrackMouseDown);
        this.refs.barHorizontal.addEventListener('mousedown', this.handleHorizontalTrackMouseDown);
        this.refs.thumbVertical.addEventListener('mousedown', this.handleVerticalThumbMouseDown);
        this.refs.thumbHorizontal.addEventListener('mousedown', this.handleHorizontalThumbMouseDown);
        document.addEventListener('mouseup', this.handleDocumentMouseUp);
        window.addEventListener('resize', this.handleWindowResize);
    },
    removeListeners: function removeListeners() {
        if (typeof document === 'undefined') return;
        this.refs.view.removeEventListener('scroll', this.handleScroll);
        this.refs.barVertical.removeEventListener('mousedown', this.handleVerticalTrackMouseDown);
        this.refs.barHorizontal.removeEventListener('mousedown', this.handleHorizontalTrackMouseDown);
        this.refs.thumbVertical.removeEventListener('mousedown', this.handleVerticalThumbMouseDown);
        this.refs.thumbHorizontal.removeEventListener('mousedown', this.handleHorizontalThumbMouseDown);
        document.removeEventListener('mouseup', this.handleDocumentMouseUp);
        window.removeEventListener('resize', this.handleWindowResize);
    },
    handleScroll: function handleScroll(event) {
        var onScroll = this.props.onScroll;

        this.update(function (values) {
            if (!onScroll) return;
            onScroll(event, values);
        });
    },
    handleVerticalTrackMouseDown: function handleVerticalTrackMouseDown(event) {
        var _refs = this.refs;
        var thumbVertical = _refs.thumbVertical;
        var barVertical = _refs.barVertical;
        var view = _refs.view;

        var offset = Math.abs(event.target.getBoundingClientRect().top - event.clientY);
        var thumbHalf = thumbVertical.offsetHeight / 2;
        var thumbPositionPercentage = (offset - thumbHalf) * 100 / barVertical.offsetHeight;
        view.scrollTop = thumbPositionPercentage * view.scrollHeight / 100;
    },
    handleHorizontalTrackMouseDown: function handleHorizontalTrackMouseDown() {
        var _refs2 = this.refs;
        var thumbHorizontal = _refs2.thumbHorizontal;
        var barHorizontal = _refs2.barHorizontal;
        var view = _refs2.view;

        var offset = Math.abs(event.target.getBoundingClientRect().left - event.clientX);
        var thumbHalf = thumbHorizontal.offsetWidth / 2;
        var thumbPositionPercentage = (offset - thumbHalf) * 100 / barHorizontal.offsetWidth;
        view.scrollLeft = thumbPositionPercentage * view.scrollWidth / 100;
    },
    handleVerticalThumbMouseDown: function handleVerticalThumbMouseDown(event) {
        this.handleDragStart(event);
        var currentTarget = event.currentTarget;
        var clientY = event.clientY;

        this.prevPageY = currentTarget.offsetHeight - (clientY - currentTarget.getBoundingClientRect().top);
    },
    handleHorizontalThumbMouseDown: function handleHorizontalThumbMouseDown(event) {
        this.handleDragStart(event);
        var currentTarget = event.currentTarget;
        var clientX = event.clientX;

        this.prevPageX = currentTarget.offsetWidth - (clientX - currentTarget.getBoundingClientRect().left);
    },
    handleDocumentMouseUp: function handleDocumentMouseUp() {
        this.handleDragEnd();
    },
    handleDocumentMouseMove: function handleDocumentMouseMove(event) {
        if (this.cursorDown === false) return false;
        if (this.prevPageY) {
            var _refs3 = this.refs;
            var barVertical = _refs3.barVertical;
            var thumbVertical = _refs3.thumbVertical;
            var view = _refs3.view;

            var offset = (barVertical.getBoundingClientRect().top - event.clientY) * -1;
            var thumbClickPosition = thumbVertical.offsetHeight - this.prevPageY;
            var thumbPositionPercentage = (offset - thumbClickPosition) * 100 / barVertical.offsetHeight;
            view.scrollTop = thumbPositionPercentage * view.scrollHeight / 100;
            return false;
        }
        if (this.prevPageX) {
            var _refs4 = this.refs;
            var barHorizontal = _refs4.barHorizontal;
            var thumbHorizontal = _refs4.thumbHorizontal;
            var view = _refs4.view;

            var offset = (barHorizontal.getBoundingClientRect().left - event.clientX) * -1;
            var thumbClickPosition = thumbHorizontal.offsetWidth - this.prevPageX;
            var thumbPositionPercentage = (offset - thumbClickPosition) * 100 / barHorizontal.offsetWidth;
            view.scrollLeft = thumbPositionPercentage * view.scrollWidth / 100;
            return false;
        }
    },
    handleWindowResize: function handleWindowResize() {
        this.update();
    },
    handleDragStart: function handleDragStart(event) {
        if (!document) return;
        event.stopImmediatePropagation();
        this.cursorDown = true;
        (0, _domCss2.default)(document.body, _styles.disableSelectStyle);
        document.addEventListener('mousemove', this.handleDocumentMouseMove);
        document.onselectstart = _returnFalse2.default;
    },
    handleDragEnd: function handleDragEnd() {
        if (!document) return;
        this.cursorDown = false;
        this.prevPageX = this.prevPageY = 0;
        (0, _domCss2.default)(document.body, _styles.resetDisableSelectStyle);
        document.removeEventListener('mousemove', this.handleDocumentMouseMove);
        document.onselectstart = undefined;
    },
    raf: function raf(callback) {
        var _this = this;

        if (this.timer) _raf3.default.cancel(this.timer);
        this.timer = (0, _raf3.default)(function () {
            _this.timer = undefined;
            callback();
        });
    },
    update: function update(callback) {
        var _refs5 = this.refs;
        var thumbHorizontal = _refs5.thumbHorizontal;
        var thumbVertical = _refs5.thumbVertical;
        var barHorizontal = _refs5.barHorizontal;
        var barVertical = _refs5.barVertical;

        var _getInnerSizePercenta = this.getInnerSizePercentage();

        var widthPercentageInner = _getInnerSizePercenta.widthPercentageInner;
        var heightPercentageInner = _getInnerSizePercenta.heightPercentageInner;

        var _getPosition = this.getPosition();

        var x = _getPosition.x;
        var y = _getPosition.y;

        var values = _objectWithoutProperties(_getPosition, ['x', 'y']);

        this.raf(function () {
            if ((0, _getScrollbarWidth2.default)() > 0) {
                var thumbHorizontalStyle = {
                    width: widthPercentageInner < 100 ? widthPercentageInner + '%' : 0,
                    transform: 'translateX(' + x + '%)'
                };
                var thumbVerticalStyle = {
                    height: heightPercentageInner < 100 ? heightPercentageInner + '%' : 0,
                    transform: 'translateY(' + y + '%)'
                };
                (0, _domCss2.default)(thumbHorizontal, thumbHorizontalStyle);
                (0, _domCss2.default)(thumbVertical, thumbVerticalStyle);

                // Hide scrollbars, when view does not overflow container
                if (widthPercentageInner >= 100) (0, _domCss2.default)(barHorizontal, { height: 0 });
                if (heightPercentageInner >= 100) (0, _domCss2.default)(barVertical, { width: 0 });
            }
            if (typeof callback !== 'function') return;
            callback(values);
        });
    },
    render: function render() {
        var scrollbarWidth = (0, _getScrollbarWidth2.default)();
        var _props = this.props;
        var style = _props.style;
        var renderScrollbarHorizontal = _props.renderScrollbarHorizontal;
        var renderScrollbarVertical = _props.renderScrollbarVertical;
        var renderThumbHorizontal = _props.renderThumbHorizontal;
        var renderThumbVertical = _props.renderThumbVertical;
        var renderView = _props.renderView;
        var onScroll = _props.onScroll;
        var children = _props.children;

        var props = _objectWithoutProperties(_props, ['style', 'renderScrollbarHorizontal', 'renderScrollbarVertical', 'renderThumbHorizontal', 'renderThumbVertical', 'renderView', 'onScroll', 'children']);

        var containerStyle = _extends({
            position: 'relative',
            overflow: 'hidden',
            width: '100%',
            height: '100%'
        }, style);

        var viewStyle = scrollbarWidth > 0 ? _extends({}, _styles.scrollbarsVisibleViewStyle, {
            right: -scrollbarWidth,
            bottom: -scrollbarWidth
        }) : _styles.scrollbarsInvisibleViewStyle;

        var finalScrollbarHorizontalStyle = scrollbarWidth > 0 ? _styles.defaultScrollbarHorizontalStyle : _extends({}, _styles.defaultScrollbarHorizontalStyle, { display: 'none' });

        var finalScrollbarVerticalStyle = scrollbarWidth > 0 ? _styles.defaultScrollbarVerticalStyle : _extends({}, _styles.defaultScrollbarVerticalStyle, { display: 'none' });

        return _react2.default.createElement(
            'div',
            _extends({}, props, { style: containerStyle }),
            (0, _react.cloneElement)(renderView({ style: viewStyle }), { ref: 'view' }, children),
            (0, _react.cloneElement)(renderScrollbarHorizontal({ style: finalScrollbarHorizontalStyle }), { ref: 'barHorizontal' }, (0, _react.cloneElement)(renderThumbHorizontal({ style: _styles.defaultThumbHorizontalStyle }), { ref: 'thumbHorizontal' })),
            (0, _react.cloneElement)(renderScrollbarVertical({ style: finalScrollbarVerticalStyle }), { ref: 'barVertical' }, (0, _react.cloneElement)(renderThumbVertical({ style: _styles.defaultThumbVerticalStyle }), { ref: 'thumbVertical' }))
        );
    }
});